@@ -607,6 +607,34 @@ class ExtractMethodTest(unittest.TestCase):
         """)
         self.assertEqual(expected, refactored)
 
+    def test_single_line_extract_method_with_large_multiline_expression(self):
+        code = dedent("""\
+            a_var = func(
+                {
+                    "hello": 1,
+                    "world": 2,
+                },
+                blah=foo,
+            )
+        """)
+        start = code.index("{") - 1
+        end = code.index("}") + 1
+        refactored = self.do_extract_method(code, start, end, "new_func")
+        expected = dedent("""\
+
+            def new_func():
+                return {
+                    "hello": 1,
+                    "world": 2,
+                }
+
+            a_var = func(
+                new_func(),
+                blah=foo,
+            )
+        """)
+        self.assertEqual(expected, refactored)
+
     def test_single_line_extract_method(self):
         code = dedent("""\
             class AClass(object):
@@ -1942,6 +1970,26 @@ class ExtractMethodTest(unittest.TestCase):
         """)
         self.assertEqual(expected, refactored)
 
+    def test_extract_method_with_list_comprehension_multiple_targets(self):
+        code = dedent("""\
+            def foo():
+                x = [(a, b) for a, b in []]
+                f = 23
+                print("hello")
+        """)
+        start, end = self._convert_line_range_to_offset(code, 4, 4)
+        refactored = self.do_extract_method(code, start, end, "baz")
+        expected = dedent("""\
+            def foo():
+                x = [(a, b) for a, b in []]
+                f = 23
+                baz()
+
+            def baz():
+                print("hello")
+        """)
+        self.assertEqual(expected, refactored)
+
     def test_extract_function_with_for_else_statemant(self):
         code = dedent("""\
             def a_func():
@@ -2962,7 +3010,7 @@ class ExtractMethodTest(unittest.TestCase):
                     print(file1)
         """)
         start, end = self._convert_line_range_to_offset(code, 2, 3)
-        refactored = self.do_extract_method(code, start, end, 'extracted', global_=True)
+        refactored = self.do_extract_method(code, start, end, "extracted", global_=True)
         expected = dedent("""\
             async def afunc():
                 extracted()
@@ -2981,7 +3029,7 @@ class ExtractMethodTest(unittest.TestCase):
                     print(file1, file2)
         """)
         start, end = self._convert_line_range_to_offset(code, 3, 3)
-        refactored = self.do_extract_method(code, start, end, 'extracted', global_=True)
+        refactored = self.do_extract_method(code, start, end, "extracted", global_=True)
         expected = dedent("""\
             async def afunc():
                 async with open("test") as file1, open("test") as file2:
@@ -2991,3 +3039,69 @@ class ExtractMethodTest(unittest.TestCase):
                 print(file1, file2)
         """)
         self.assertEqual(expected, refactored)
+
+    @testutils.only_for_versions_higher("3.10")
+    def test_extract_method_containing_structural_pattern_match(self):
+        code = dedent("""\
+            match var:
+                case Foo("xx"):
+                    print(x)
+                case Foo(x):
+                    print(x)
+        """)
+        start, end = self._convert_line_range_to_offset(code, 5, 5)
+        refactored = self.do_extract_method(code, start, end, "extracted")
+        expected = dedent("""\
+
+            def extracted():
+                print(x)
+
+            match var:
+                case Foo("xx"):
+                    print(x)
+                case Foo(x):
+                    extracted()
+        """)
+        self.assertEqual(expected, refactored)
+
+    @testutils.only_for_versions_higher("3.10")
+    def test_extract_method_containing_structural_pattern_match_2(self):
+        code = dedent("""\
+            def foo():
+                match var:
+                    case Foo(x):
+                        print(x)
+        """)
+        start, end = self._convert_line_range_to_offset(code, 4, 4)
+        refactored = self.do_extract_method(code, start, end, "extracted")
+        expected = dedent("""\
+            def foo():
+                match var:
+                    case Foo(x):
+                        extracted(x)
+
+            def extracted(x):
+                print(x)
+        """)
+        self.assertEqual(expected, refactored)
+
+    @testutils.only_for_versions_higher("3.10")
+    def test_extract_method_containing_structural_pattern_match_3(self):
+        code = dedent("""\
+            def foo():
+                match var:
+                    case {"hello": x} as y:
+                        print(x)
+        """)
+        start, end = self._convert_line_range_to_offset(code, 4, 4)
+        refactored = self.do_extract_method(code, start, end, "extracted")
+        expected = dedent("""\
+            def foo():
+                match var:
+                    case {"hello": x} as y:
+                        extracted(x)
+
+            def extracted(x):
+                print(x)
+        """)
+        self.assertEqual(expected, refactored)
